import ipaddress
import json
import re
import socket
import sys
from collections import defaultdict

import requests

AWS_IP_RANGES_URL = 'https://ip-ranges.amazonaws.com/ip-ranges.json'


def get_range_prefixes():
    try:
        data = requests.get(AWS_IP_RANGES_URL, timeout=10).json()
    except Exception:
        raise SystemExit('Failed to get IP ranges from AWS')
    else:
        return data['prefixes']


def resolve(hostname):
    try:
        return socket.gethostbyname(hostname)
    except socket.gaierror:
        return False


def is_ip(string):
    try:
        return ipaddress.ip_address(string)
    except ValueError:
        return False


def find_prefix(prefixes, ip):
    amz_prefix = None
    non_amz_prefix = None

    # We probably can't rely on the order of prefixes being alphabetical.
    # So try to find any non-AMAZON services with higher precedence.
    # If none found, return the AMAZON service.
    for prefix in prefixes:
        subnet = ipaddress.ip_network(prefix['ip_prefix'])

        if ip in subnet:
            if prefix['service'] == 'AMAZON':
                amz_prefix = prefix
            else:
                non_amz_prefix = prefix
                break

    if non_amz_prefix:
        return non_amz_prefix

    return amz_prefix


def generate_response(result, ip_address=None, hostname=None):
    if result:
        response = {
            'is_aws_ip': True,
            'ip_address': ip_address,
            'service': result['service'],
            'region': result['region'],
            'matched_subnet': result['ip_prefix']
        }
        if hostname:
            response['hostname'] = hostname
    else:
        response = {'is_aws_ip': False}
        
    return response


def process_one(prefixes, host):
    if is_ip(host):
        result = find_prefix(prefixes, ipaddress.ip_address(host))
        response = generate_response(result, ip_address=host)
        return response
    else:
        ip_address = resolve(host)
        if ip_address:
            result = find_prefix(prefixes, ipaddress.ip_address(ip_address))
            response = generate_response(result, hostname=host, ip_address=ip_address)
            return response
        else:
            return {'resolvable': False}


def process(prefixes, args):
    if args['input']:
        return json.dumps(process_one(prefixes, args['input']), indent=4)

    results = defaultdict()
    for host in args['hosts']:
        print(f'Processing: {host}')
        results[host] = process_one(prefixes, host)
    return json.dumps(dict(results), indent=4)


def run(prefixes, args):
    # print(args['hosts'])
    results = process(prefixes, args)
    print(results)



import argparse
import sys
from argparse import RawDescriptionHelpFormatter

spec = {
    'description': 'Check if a hostname/IP belongs to AWS',
    'arguments': [
        {
            'options': ['input'],
            'action': 'store',
            'help': 'Input hostname / IP',
            'nargs': '?'
        }
    ],
    'epilog': "Examples:\n\
        onaws 52.219.47.34\n\
        onaws flaws.cloud\n\
        onaws uber.s3.amazonaws.com\n",
    'formatter_class': RawDescriptionHelpFormatter
}

def create_parser(spec):
    parser = argparse.ArgumentParser(
        description=spec['description'],
        epilog=spec['epilog'],
        formatter_class=spec['formatter_class']
    )

    for arg in spec['arguments']:
        options = arg.pop('options', ())
        parser.add_argument(*options, **arg)    
    
    return parser

parser = create_parser(spec)


def get_raw_input(input_str):
    if input_str is None:
        return sys.stdin.read()

    return input_str


def parse_raw_input(text):
    return [line.strip() for line in text.strip().splitlines() if line.strip()]


def gather_input(args):
    raw = get_raw_input(args.input)
    hosts = parse_raw_input(raw)
    return {
        'input': args.input,
        'hosts': hosts,
    }



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['onaws']

package_data = \
{'': ['*']}

install_requires = \
['requests']

entry_points = \
{'console_scripts': ['onaws = onaws:__main__.main']}

setup(name='onaws',
      version='0.0.12',
      description='Simple library to check if a hostname belongs to AWS IP space.',
      author='Amal Murali',
      author_email='amalmurali47@gmail.com',
      url='https://github.com/amalmurali47/onaws',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



from . import cli, core

def main():
    args = cli.parser.parse_args()
    args = cli.gather_input(args)

    prefixes = core.get_range_prefixes()

    core.run(prefixes, args)


if __name__ == "__main__":
    main()